"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useTransition = exports.useCSSTransition = void 0;
const react_1 = require("react");
const state_js_1 = require("./state.js");
const useForceUpdate_js_1 = __importDefault(require("./useForceUpdate.js"));
__exportStar(require("./types.js"), exports);
const IMMEDIATE_UPDATE = 0;
/**
 * @param items - the items you want to animate.  If an item leaves this list,
 *   it will temporarily be kept in the returned results until it has time to
 *   transition out.
 * @param getKey - A function used to get a unique key for each item in `items`.
 * @param config - Styles to apply to item.  `from` is applied to a new
 *   item when that item first is added to `items`.  `initial` is applied to
 *   items on the very first render.  `from` is applied to an item when it
 *   first is added to `items`.  `enter` is applied to items while they are
 *   entering.  `update` is applied to an item that has entered (if `update`
 *   is not specified, `enter` is used by default), `leave` is applied to items
 *   that are leaving.  In addition `enterTime` and `leaveTime` specify the duration,
 *   in milliseconds, that these transitions should take.  `common` is a set of
 *   properties that will be merged in to all other states - handy for setting
 *   "transition" and "will-change".
 */
function useCSSTransition(items, getKey, config) {
    return useTransition(items, getKey, config);
}
exports.useCSSTransition = useCSSTransition;
/**
 * This is like `useCSSTransition()`, except you can pass in any props you want
 * instead of just CSS properties.  This is most useful when you want to construct
 * two different styles for related objects (like you want to scale and position
 * and image, and want to position but not scale text over top of the image).
 *
 * This does *not* do any kind of interpolation - you have to take care of that
 * part yourself (or let your browser's CSS engine take care of it, ideally).
 *
 * @param items - the items you want to animate.  If an item leaves this list,
 *   it will temporarily be kept in the returned results until it has time to
 *   transition out.
 * @param getKey - A function used to get a unique key for each item in `items`.
 * @param config - Styles to apply to item.  `from` is applied to a new
 *   item when that item first is added to `items`.  `initial` is applied to
 *   items on the very first render.  `from` is applied to an item when it
 *   first is added to `items`.  `enter` is applied to items while they are
 *   entering.  `update` is applied to an item that has entered (if `update`
 *   is not specified, `enter` is used by default), `leave` is applied to items
 *   that are leaving.  In addition `enterTime` and `leaveTime` specify the duration,
 *   in milliseconds, that these transitions should take.  `common` is a set of
 *   properties that will be merged in to all other states - handy for setting
 *   "transition" and "will-change".
 */
function useTransition(items, getKey, config) {
    const forceUpdate = useForceUpdate_js_1.default();
    const state = react_1.useRef();
    const nextState = state.current
        ? state_js_1.generateNextState(state.current, items, getKey, config)
        : state_js_1.generateInitialState(items, getKey, config);
    state.current = nextState;
    const nextUpdate = state.current.nextUpdate;
    react_1.useEffect(() => {
        let cleanup;
        if (nextUpdate !== undefined) {
            if (nextUpdate === IMMEDIATE_UPDATE || nextUpdate <= Date.now()) {
                forceUpdate();
            }
            else {
                const timeTillUpdate = nextUpdate - Date.now();
                const timer = setTimeout(forceUpdate, timeTillUpdate);
                cleanup = () => clearTimeout(timer);
            }
        }
        return cleanup;
    }, [nextUpdate, forceUpdate]);
    return nextState.items;
}
exports.useTransition = useTransition;
exports.default = useCSSTransition;
