"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
/**
 * Returns a function which can be called to force the parent component to update.
 */
function useForceUpdate() {
    const [, setCount] = react_1.useState(0);
    const updating = react_1.useRef(false);
    let defer;
    if (typeof window !== 'undefined') {
        defer = window.requestAnimationFrame;
    }
    else {
        defer = (fn) => setTimeout(fn, 1);
    }
    return react_1.useCallback(() => {
        if (updating.current) {
            // Already updating.
            return;
        }
        updating.current = true;
        defer(() => {
            updating.current = false;
            setCount((count) => {
                if (count === Number.MAX_SAFE_INTEGER) {
                    return 0;
                }
                else {
                    return count + 1;
                }
            });
        });
    }, [defer]);
}
exports.default = useForceUpdate;
