import { Config, TransitionItem } from './types.js';
export * from './types.js';
/**
 * @param items - the items you want to animate.  If an item leaves this list,
 *   it will temporarily be kept in the returned results until it has time to
 *   transition out.
 * @param getKey - A function used to get a unique key for each item in `items`.
 * @param config - Styles to apply to item.  `from` is applied to a new
 *   item when that item first is added to `items`.  `initial` is applied to
 *   items on the very first render.  `from` is applied to an item when it
 *   first is added to `items`.  `enter` is applied to items while they are
 *   entering.  `update` is applied to an item that has entered (if `update`
 *   is not specified, `enter` is used by default), `leave` is applied to items
 *   that are leaving.  In addition `enterTime` and `leaveTime` specify the duration,
 *   in milliseconds, that these transitions should take.  `common` is a set of
 *   properties that will be merged in to all other states - handy for setting
 *   "transition" and "will-change".
 */
export declare function useCSSTransition<T>(items: readonly T[], getKey: (item: T) => string | number, config: Config<T>): TransitionItem<T>[];
/**
 * This is like `useCSSTransition()`, except you can pass in any props you want
 * instead of just CSS properties.  This is most useful when you want to construct
 * two different styles for related objects (like you want to scale and position
 * and image, and want to position but not scale text over top of the image).
 *
 * This does *not* do any kind of interpolation - you have to take care of that
 * part yourself (or let your browser's CSS engine take care of it, ideally).
 *
 * @param items - the items you want to animate.  If an item leaves this list,
 *   it will temporarily be kept in the returned results until it has time to
 *   transition out.
 * @param getKey - A function used to get a unique key for each item in `items`.
 * @param config - Styles to apply to item.  `from` is applied to a new
 *   item when that item first is added to `items`.  `initial` is applied to
 *   items on the very first render.  `from` is applied to an item when it
 *   first is added to `items`.  `enter` is applied to items while they are
 *   entering.  `update` is applied to an item that has entered (if `update`
 *   is not specified, `enter` is used by default), `leave` is applied to items
 *   that are leaving.  In addition `enterTime` and `leaveTime` specify the duration,
 *   in milliseconds, that these transitions should take.  `common` is a set of
 *   properties that will be merged in to all other states - handy for setting
 *   "transition" and "will-change".
 */
export declare function useTransition<T, P>(items: readonly T[], getKey: (item: T) => string | number, config: Config<T, P>): TransitionItem<T, P>[];
export default useCSSTransition;
