import { useCallback, useRef, useState } from 'react';
/**
 * Returns a function which can be called to force the parent component to update.
 */
export default function useForceUpdate() {
    const [, setCount] = useState(0);
    const updating = useRef(false);
    let defer;
    if (typeof window !== 'undefined') {
        defer = window.requestAnimationFrame;
    }
    else {
        defer = (fn) => setTimeout(fn, 1);
    }
    return useCallback(() => {
        if (updating.current) {
            // Already updating.
            return;
        }
        updating.current = true;
        defer(() => {
            updating.current = false;
            setCount((count) => {
                if (count === Number.MAX_SAFE_INTEGER) {
                    return 0;
                }
                else {
                    return count + 1;
                }
            });
        });
    }, [defer]);
}
//# sourceMappingURL=useForceUpdate.js.map